/*
 * (C) 2008 by OpenMoko, Inc.
 *  Written by smurfy - phil (openmoko@smurfy.de)
 *
 *  simple tool to use FreeRunner LEDs
 *  some code copied from neod =)
 *
 *  blinking rules 90% like: 
 *  http://wiki.openmoko.org/wiki/FreeRunner/Buttons_and_LEDs
 *
 *  version 0.1
 *
 *  ToDo:
 *  - Event stuff (Aux Led)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 *  See the *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <termios.h> 
#include <string.h>
#include <time.h>
#include <netdb.h>
#include <linux/if.h>
#include <linux/wireless.h>
#include <apm.h>

#define SYS_CLASS_BT_STATE "/sys/bus/platform/devices/neo1973-pm-bt.0/power_on"
#define WIFI_IFACE "eth0"
#define SYS_CLASS_LED_EVENT "/sys/devices/platform/gta02-led.0/leds/gta02-aux:red/brightness"
#define SYS_CLASS_LED_HEARTBEAT "/sys/devices/platform/gta02-led.0/leds/gta02-power:orange/brightness"
#define SYS_CLASS_LED_WIRELESS "/sys/devices/platform/gta02-led.0/leds/gta02-power:blue/brightness"


void write_boolean_to_path( const char* path, int b )
{
    FILE* f = fopen( path, "w" );
    if ( f == NULL )
    {
        return;
    }
    fprintf( f, b ? "1\n" : "0\n" );
    fclose( f );
}

int read_boolean_from_path( const char* path )
{
    int value;
    FILE* f = fopen( path, "r" );
    if ( f == NULL )
    {
        return 0;
    }
    fscanf( f, "%d", &value );
    fclose( f );
    return value;
}

int wifi_radio_is_on (const char *iface)
{
  struct iwreq wrq;
  int sock = 0; /* socket */

  /* Open socket to perform ioctl() on */
  sock = socket (AF_INET, SOCK_DGRAM, 0);
  if (!sock)
  {
    return 0;
  } 

 /* Clear our request and set the interface name */
  memset (&wrq, 0, sizeof (struct iwreq));
  strncpy ((char *)&wrq.ifr_name, iface, IFNAMSIZ);

  /* Feel the power, uhh, do the ioctl() */
  if (ioctl (sock, SIOCGIWTXPOW, &wrq) != 0)
  {
    close (sock);
    return 0;
  }

  close (sock);

  return !wrq.u.txpower.disabled; 

}

int main (int argc, char *argv[])
{	
	puts ("OpenMoko simple Led display");
	int i_blink_count = 0;
	int b_hb = 0; 
	while(1) {
		/* check battery */
		if (i_blink_count == 0) {
			apm_info info;
    			memset (&info, 0, sizeof (apm_info));
			apm_read(&info);
			if ( info.battery_status == BATTERY_STATUS_ABSENT ||
			      info.battery_status == BATTERY_STATUS_CHARGING )
			{
				if (info.battery_percentage < 10)
				   b_hb = 1; 
				else if (info.battery_percentage < 30)
				   b_hb = 2; 
				else if (info.battery_percentage < 50)
				   b_hb = 3; 
				else if (info.battery_percentage < 70)
				   b_hb = 4; 
				else if (info.battery_percentage < 90)
				   b_hb = 5; 
				else
				   b_hb = 9; 
			} else {	
				/* normal heartbeat */
				b_hb = 0;
			}
		}

		
		/* check bluetooth and wifi */
		int b_bt = read_boolean_from_path(SYS_CLASS_BT_STATE);
		int b_wifi = wifi_radio_is_on(WIFI_IFACE);
		if ((b_bt == 1) && b_wifi == 1) {
			/* bt and wifi is on. wireless should lit all the time */
			write_boolean_to_path(SYS_CLASS_LED_WIRELESS,1);
		} 
		if ((b_bt == 1) && b_wifi == 0) {
			/* bt is on only.. fast blink (~200ms) every 2 sek*/
			if ((i_blink_count == 0) || (i_blink_count == 9))  {
				write_boolean_to_path(SYS_CLASS_LED_WIRELESS,1);
			}
			if ((i_blink_count == 1) || (i_blink_count == 10))  {
				write_boolean_to_path(SYS_CLASS_LED_WIRELESS,0);
			}
		}
		if ((b_bt == 0) && b_wifi == 1) {
			/* wifi is on only.. slow blink (~400 ms) every 2 sek*/
			if ((i_blink_count == 0) || (i_blink_count == 9))  {
				write_boolean_to_path(SYS_CLASS_LED_WIRELESS,1);
			}
			if ((i_blink_count == 2) || (i_blink_count == 11))  {
				write_boolean_to_path(SYS_CLASS_LED_WIRELESS,0);
			}
		}


		/* normal heartbeat a 400ms blink every ~4 sec*/ 
		if (b_hb != 9) {
			if ((i_blink_count == 5) || (i_blink_count == 6)) {
				write_boolean_to_path(SYS_CLASS_LED_HEARTBEAT,1);
				if (read_boolean_from_path(SYS_CLASS_LED_WIRELESS) == 1) {
					write_boolean_to_path(SYS_CLASS_LED_WIRELESS,0);
				}
			}
			if (i_blink_count == 7) {
				write_boolean_to_path(SYS_CLASS_LED_HEARTBEAT,0);
			}
		}

		if ((b_hb > 3) && (b_hb != 9)) {
			if ((i_blink_count == 12) || (i_blink_count == 13)) {
				write_boolean_to_path(SYS_CLASS_LED_HEARTBEAT,1);
				if (read_boolean_from_path(SYS_CLASS_LED_WIRELESS) == 1) {
					write_boolean_to_path(SYS_CLASS_LED_WIRELESS,0);
				}
			}
			if (i_blink_count == 14) {
				write_boolean_to_path(SYS_CLASS_LED_HEARTBEAT,0);
			}
		}

		/* lit alwys */
		if (b_hb == 9) {
			write_boolean_to_path(SYS_CLASS_LED_HEARTBEAT,1);
		}

		/* sleeping 400 ms */
		usleep(400000);

		i_blink_count++;
		if (i_blink_count >= 20) {
			i_blink_count = 0;
		} 
	}

	write_boolean_to_path(SYS_CLASS_LED_HEARTBEAT,0);
	write_boolean_to_path(SYS_CLASS_LED_EVENT,0);
	write_boolean_to_path(SYS_CLASS_LED_WIRELESS,0);

	return 0;
}

